@@ -49,16 +49,14 @@ class MDD(BaseAdaptDeep):
         
     Examples
     --------
-    >>> Xs = np.concatenate((np.random.random((100, 1)),
-    ...                      np.zeros((100, 1))), 1)
-    >>> Xt = np.concatenate((np.random.random((100, 1)),
-    ...                      np.ones((100, 1))), 1)
-    >>> ys = 0.2 * Xs[:, 0]
-    >>> yt = 0.2 * Xt[:, 0]
-    >>> model = MDD(random_state=0)
-    >>> model.fit(Xs, ys, Xt, yt, epochs=100, verbose=0)
-    >>> model.history_["task_t"][-1]
-    0.0009...
+    >>> from adapt.utils import make_classification_da
+    >>> from adapt.feature_based import MDD
+    >>> Xs, ys, Xt, yt = make_classification_da()
+    >>> model = MDD(lambda_=0.1, gamma=4., Xt=Xt, metrics=["acc"], random_state=0)
+    >>> model.fit(Xs, ys, epochs=100, verbose=0)
+    >>> model.score(Xt, yt)
+    1/1 [==============================] - 0s 102ms/step - loss: 0.1971 - acc: 0.7800
+    0.19707153737545013
         
     References
     ----------
